/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/signals.js TAP darwin > must match snapshot 1`] = `
Object {
  "signals": Array [
    "SIGHUP",
    "SIGINT",
    "SIGTERM",
    "SIGALRM",
    "SIGABRT",
    "SIGVTALRM",
    "SIGXCPU",
    "SIGXFSZ",
    "SIGUSR2",
    "SIGTRAP",
    "SIGSYS",
    "SIGQUIT",
    "SIGIOT",
  ],
}
`

exports[`test/signals.js TAP linux > must match snapshot 1`] = `
Object {
  "signals": Array [
    "SIGHUP",
    "SIGINT",
    "SIGTERM",
    "SIGALRM",
    "SIGABRT",
    "SIGVTALRM",
    "SIGXCPU",
    "SIGXFSZ",
    "SIGUSR2",
    "SIGTRAP",
    "SIGSYS",
    "SIGQUIT",
    "SIGIOT",
    "SIGIO",
    "SIGPOLL",
    "SIGPWR",
    "SIGSTKFLT",
  ],
}
`

exports[`test/signals.js TAP win32 > must match snapshot 1`] = `
Object {
  "signals": Array [
    "SIGHUP",
    "SIGINT",
    "SIGTERM",
  ],
}
`
